// Copyright (c) 2024 FWORKS. All Rights Reserved.

#include "Demo/SeasonsFSMComponent.h"
#include "Demo/SuperSimpleFSMDemoTags.h"
#include "UnrealEngine.h"
#include "TimerManager.h"
#include "Components/TextRenderComponent.h"

// Sets default values for this component's properties
USeasonsFSMComponent::USeasonsFSMComponent()
{
	InitialState = SuperSimpleFSMDemoTags::TAG_SSFSM_Seasons_Spring;

    SetIsReplicatedByDefault(true);
}

void USeasonsFSMComponent::OnRegisterStateCallbacks_Implementation()
{
    FSuperSimpleFSMStateDelegate EmptyStateDelegate;
    FSuperSimpleFSMStateTickDelegate EmptyTickDelegate;

    {
        FSuperSimpleFSMStateDelegate EnterDel;
        EnterDel.BindUFunction(this, GET_FUNCTION_NAME_CHECKED(USeasonsFSMComponent, Spring_Begin));
        RegisterStateEvents(SuperSimpleFSMDemoTags::TAG_SSFSM_Seasons_Spring, EnterDel, EmptyStateDelegate, EmptyTickDelegate);
    }

    {
        FSuperSimpleFSMStateDelegate EnterDel;
        EnterDel.BindUFunction(this, GET_FUNCTION_NAME_CHECKED(USeasonsFSMComponent, SummerRainy_Begin));
        RegisterStateEvents(SuperSimpleFSMDemoTags::TAG_SSFSM_Seasons_Summer_Rainy, EnterDel, EmptyStateDelegate, EmptyTickDelegate);
    }

    {
        FSuperSimpleFSMStateDelegate EnterDel;
        EnterDel.BindUFunction(this, GET_FUNCTION_NAME_CHECKED(USeasonsFSMComponent, SummerDrought_Begin));
        RegisterStateEvents(SuperSimpleFSMDemoTags::TAG_SSFSM_Seasons_Summer_Drought, EnterDel, EmptyStateDelegate, EmptyTickDelegate);
    }

    {
        FSuperSimpleFSMStateDelegate EnterDel;
        EnterDel.BindUFunction(this, GET_FUNCTION_NAME_CHECKED(USeasonsFSMComponent, Autumn_Begin));
        RegisterStateEvents(SuperSimpleFSMDemoTags::TAG_SSFSM_Seasons_Autumn, EnterDel, EmptyStateDelegate, EmptyTickDelegate);
    }

	{
        FSuperSimpleFSMStateDelegate EnterDel;
        EnterDel.BindUFunction(this, GET_FUNCTION_NAME_CHECKED(USeasonsFSMComponent, Winter_Begin));
        FSuperSimpleFSMStateTickDelegate TickDel;
        TickDel.BindUFunction(this, GET_FUNCTION_NAME_CHECKED(USeasonsFSMComponent, Winter_Tick));
        RegisterStateEvents(SuperSimpleFSMDemoTags::TAG_SSFSM_Seasons_Winter, EnterDel, EmptyStateDelegate, TickDel);
	}

    {
        FSuperSimpleFSMStateDelegate EnterDel;
        EnterDel.BindUFunction(this, GET_FUNCTION_NAME_CHECKED(USeasonsFSMComponent, WinterSnowy_Begin));
        RegisterStateEvents(SuperSimpleFSMDemoTags::TAG_SSFSM_Seasons_Winter_Snowy, EnterDel, EmptyStateDelegate, EmptyTickDelegate);
    }

    {
        FSuperSimpleFSMStateDelegate EnterDel;
        EnterDel.BindUFunction(this, GET_FUNCTION_NAME_CHECKED(USeasonsFSMComponent, WinterDry_Begin));
        RegisterStateEvents(SuperSimpleFSMDemoTags::TAG_SSFSM_Seasons_Winter_Dry, EnterDel, EmptyStateDelegate, EmptyTickDelegate);
    }
	
}

void USeasonsFSMComponent::Spring_Begin(FGameplayTag StateTag)
{
    PrintStateDebug("Spring");

    GetWorld()->GetTimerManager().SetTimer(TimerHandle, FTimerDelegate::CreateWeakLambda(this, [this]() 
        {
            // example of transitioning to a nested state.
            // This will Exit Spring, Enter Summer, Enter Summer Drought/Rainy

            // 50% chance of rainy vs drought
            if (FMath::RandBool())
            {
                PerformTransition(SuperSimpleFSMDemoTags::TAG_SSFSM_Seasons_Summer_Drought);
            }
            else
            {
                PerformTransition(SuperSimpleFSMDemoTags::TAG_SSFSM_Seasons_Summer_Rainy);
            }
            
        }), 2.0f, false);
}

void USeasonsFSMComponent::SummerRainy_Begin(FGameplayTag StateTag)
{
    PrintStateDebug("Summer (Rainy)");

    GetWorld()->GetTimerManager().SetTimer(TimerHandle, FTimerDelegate::CreateWeakLambda(this, [this]()
    {
        PerformTransition(SuperSimpleFSMDemoTags::TAG_SSFSM_Seasons_Autumn);
    }), 2.0f, false);
}

void USeasonsFSMComponent::SummerDrought_Begin(FGameplayTag StateTag)
{
    PrintStateDebug("Summer (Drought)");

    GetWorld()->GetTimerManager().SetTimer(TimerHandle, FTimerDelegate::CreateWeakLambda(this, [this]()
    {
        PerformTransition(SuperSimpleFSMDemoTags::TAG_SSFSM_Seasons_Autumn);
    }), 2.0f, false);
}

void USeasonsFSMComponent::Autumn_Begin(FGameplayTag StateTag)
{
    PrintStateDebug("Autumn");

    GetWorld()->GetTimerManager().SetTimer(TimerHandle, FTimerDelegate::CreateWeakLambda(this, [this]()
    {
        // we will go to the winter parent state, and let that decide which child to enter
        PerformTransition(SuperSimpleFSMDemoTags::TAG_SSFSM_Seasons_Winter);
    }), 2.0f, false);
}

void USeasonsFSMComponent::Winter_Begin(FGameplayTag StateTag)
{
    if (FMath::RandBool())
    {
        PerformTransition(SuperSimpleFSMDemoTags::TAG_SSFSM_Seasons_Winter_Snowy);
    }
    else
    {
        PerformTransition(SuperSimpleFSMDemoTags::TAG_SSFSM_Seasons_Winter_Dry);
    }

    TickTimer = 0.0f;
}

void USeasonsFSMComponent::Winter_Tick(FGameplayTag StateTag, float DeltaTime)
{
    // example of a ticking state and performing a transition in tick

    TickTimer += DeltaTime;
    if (TickTimer > 2.0f)
    {
        PerformTransition(SuperSimpleFSMDemoTags::TAG_SSFSM_Seasons_Spring);
    }
}

void USeasonsFSMComponent::WinterSnowy_Begin(FGameplayTag StateTag)
{
    PrintStateDebug("Winter (Snowy)");
}

void USeasonsFSMComponent::WinterDry_Begin(FGameplayTag StateTag)
{
    PrintStateDebug("Winter (Dry)");
}

void USeasonsFSMComponent::PrintStateDebug(FString DebugText)
{
    AActor* OwnerActor = GetOwner();
    if (!OwnerActor)
    {
        return;
    }

    UTextRenderComponent* TextRenderComp = OwnerActor->GetComponentByClass<UTextRenderComponent>();
    if (TextRenderComp)
    {
        TextRenderComp->SetText(FText::FromString(DebugText));
    }
}
